/*
Copyright, 1994-1995, The Regents of the University of California. 
This software was produced under U.S. Government contract(XXXXX)
by Los Alamos National Laboratory, which is operated by the University 
of California for the U.S. Department of Energy. The U.S Government 
is licensed to use, reproduce, and distribute this software. Permission 
is granted to the public to copy and use this software without charge, 
provided that this Notice and any statement of authorship are reproduced 
on all copies. Neither the Government nor the University makes any 
warranty, express or implied, or assumes any liability or responsibility 
for the use of this software.
*/

/* $Id: calc.h,v 1.2 96/09/11 13:33:07 arve Exp $ */

#ifndef CALC
#define CALC

#include "matrix.h"
#include "seqlist.h"
#include "structures.h"

typedef enum {
    nucl_a=0,
    nucl_c,
    nucl_g,
    nucl_t,
    end_of_nucl_codes,		/* This is only a 'stop byte' */
    nucl_not_implemented		/* Will be ignored */
} nucleotide_code;	

typedef struct MATRIX_LIST {
    char *seq1, *seq2;		/* Sequences names */
    float_matrix *change_count;
    float_matrix *probability_estimate;
    unsigned long base_count[4];
    struct MATRIX_LIST *next;
} matrix_list;
/* Error codes */

 
struct collected_data {
    seqlist *entries;		/* The sequences */
    int nr_of_entries;
    matrix_list *data;		/* Pairwise data assembled from the seqs */
    float_matrix *base_count;
    float_matrix *eigenvectors;
    int steady_state_vector;
    float_matrix *rates, *distances, *distance_errors;
};


typedef enum  {
    C_NO_ERROR=0,
    C_OUT_OF_MEMORY,
    C_NOT_A_SQUARE_MATRIX,
    C_WRONG_SIZE_OF_MATRIX,
    C_NULL_MATRIX,
    C_BAD_SEQUENCES,
    C_COMPUTATIONAL_ERROR,
    C_COULD_NOT_FIND_STEADY_STATE,
    C_PROGRAMMING_ERROR,
    C_END_OF_ERRORS
} calc_errors;

extern char *calc_error_codes[];

void calc_free_matrix_list(matrix_list *ml);
matrix_list * calc_pairwise_freqs(seqlist *entries, float_matrix *count);
matrix_list * calc_specified_freqs(seqlist *entries, seqnamelist *from, seqnamelist *to, float_matrix *nucl_count);
matrix_list * calc_paired_freqs(seqlist *entries, seqnamelist *from, seqnamelist *to, seqnamelist *weights, float_matrix *nucl_count);
calc_errors differences(seqlist *e1, seqlist *e2, float_matrix *changes);

nucleotide_code nucleotide(char a);
calc_errors make_rate_matrix(float_matrix *m, unsigned long count[4]);
calc_errors symmetrize(float_matrix *m, float_matrix *nucleotide_ratios);
calc_errors average(float_matrix *m);
/*calc_errors eigenvects(float_matrix *m, float_matrix *eigenvects, float_matrix *eigenvals);*/

calc_errors
calc_compute_bruno(struct collected_data *d, 
		   float_matrix **distance_estimates, 
		   float_matrix **error_estimates, 
		   float_matrix **substitution_rate_matrix,
		   FILE* info_file);


calc_errors
extract_and_transform_eigens(float_matrix *eigenvectors, 
			     float_matrix *e_vectors[end_of_nucl_codes], 
			     float_matrix *e_vectors_adjoint[end_of_nucl_codes], 
			     float_matrix *ratios);
calc_errors
distance_variance(float distance[3], 
		  float_matrix *left_eigen[4], 
		  float_matrix *right_eigen[4], 
		  float_matrix *eigenvalues, 
		  float_matrix *substitution_probability, 
		  unsigned long base_count[4], 
		  int steady_state_vector,
		  float_matrix *return_values);

calc_errors
distance_covariance(float distance[3],
		    float_matrix *left_eigen[4],
		    float_matrix *right_eigen[4],
		    float_matrix *eigenvalues,
		    float_matrix *substitution_probability,
		    unsigned long base_count[4], 
		    int steady_state_vector,
		    float_matrix *return_values);

calc_errors
least_square_distance(float distance[3], 
		      float_matrix *inv_covariance, 
		      float *estimate);
 
calc_errors
sqrt_distance_variance(float_matrix *inv_covariance,
		       float *error_estimate);

calc_errors
find_substitution_rate_matrix(float_matrix *eigenvalues,
			      float_matrix *equilibrium_matrix,
			      int steady_state_vector, 
			      float_matrix *estimate);

calc_errors
estimate_rates_and_distances(matrix_list *substitution_prob_list,
			     float_matrix *equilibrium_matrix, 
			     float_matrix *eigenvectors,
			     int steady_state_vector,
			     float_matrix *distance_matrix,
			     float_matrix *error_matrix,
			     float_matrix *substitution_rate_matrix);

int find_steady_state_vector(float_matrix *eigenvectors);

void print_calc_error(calc_errors error_code);
char * calc_error_string(calc_errors error_code);


#endif 
//CALC

