/*
Copyright, 1994-1995, The Regents of the University of California. 
This software was produced under U.S. Government contract(XXXXX)
by Los Alamos National Laboratory, which is operated by the University 
of California for the U.S. Department of Energy. The U.S Government 
is licensed to use, reproduce, and distribute this software. Permission 
is granted to the public to copy and use this software without charge, 
provided that this Notice and any statement of authorship are reproduced 
on all copies. Neither the Government nor the University makes any 
warranty, express or implied, or assumes any liability or responsibility 
for the use of this software.
*/

/* $Id: matrix.h,v 1.2 96/09/11 13:33:29 arve Exp $ */

#ifndef MATRIX
#define MATRIX

#include <stdio.h>

typedef struct FLOAT_MATRIX {
  unsigned size;		/* -1 when rows != columns */
  unsigned rows, columns;
  float *matrix;
  float max_val;
  float min_val;
} float_matrix;

typedef enum  {
  M_NO_ERROR=0,
  M_OUT_OF_MEMORY,
  M_NULL_MATRIX,
  M_OUT_OF_BOUNDS,
  M_WRONG_SIZE_OF_MATRIX,
  M_END_OF_ERRORS
} matrix_errors; 
/* If you add anything here, don't forget to add 
 * an error string in matrix.c too! 
 */

extern char *matrix_error_codes[];

float_matrix * matrix_init(unsigned rows, unsigned columns);
float_matrix * matrix_init_square(unsigned size);
matrix_errors matrix_free(float_matrix *m);

matrix_errors matrix_set(float_matrix *m, unsigned row, unsigned col, float val
);
matrix_errors matrix_set_all(float_matrix *m, float values[]);

float matrix_val(float_matrix *m, unsigned row, unsigned col);

matrix_errors matrix_copy(float_matrix *from, float_matrix *to);
matrix_errors matrix_copy_column(float_matrix *from, unsigned from_col, float_matrix *to, unsigned to_col);
matrix_errors matrix_copy_row(float_matrix *from, unsigned from_row, float_matrix *to, unsigned to_row);

matrix_errors matrix_transpose(float_matrix *m);
float_matrix * matrix_make_transpose(float_matrix *m);

matrix_errors matrix_normalize_vector(float_matrix *v);
matrix_errors matrix_normalize_column(float_matrix *m, unsigned i);

int matrix_size(float_matrix *m);
int matrix_rows(float_matrix *m);
int matrix_columns(float_matrix *m);

float matrix_min(float_matrix *m);
float matrix_max(float_matrix *m);
float matrix_val_span(float_matrix *m);

matrix_errors matrix_set_min(float_matrix *m, float min);
matrix_errors matrix_set_max(float_matrix *m, float max);

matrix_errors matrix_add_to(float_matrix *a, float_matrix *sum);
matrix_errors matrix_mul(float_matrix *a, float_matrix *b, float_matrix *res);
matrix_errors matrix_scalar_mul(float_matrix *m, float s);

/* 
   Please note!
   The following function only deals with 3x3 matrices! 
*/
matrix_errors matrix_inverse(float_matrix *m, float_matrix *inverse);

matrix_errors matrix_eigen(float_matrix *m, float_matrix *eigenvects, float_matrix *eigenvals);

matrix_errors matrix_fprint_row(FILE *f, float_matrix *m, int i);
matrix_errors matrix_fprint(FILE *f, float_matrix *m);

#define matrix_print(matrix) matrix_fprint(stdout, matrix)

float_matrix* matrix_read(FILE *f, unsigned rows, unsigned columns);

void print_matrix_error(matrix_errors errflag);

#endif 
//MATRIX
