/*
Copyright, 1994-1995, The Regents of the University of California. 
This software was produced under U.S. Government contract(XXXXX)
by Los Alamos National Laboratory, which is operated by the University 
of California for the U.S. Department of Energy. The U.S Government 
is licensed to use, reproduce, and distribute this software. Permission 
is granted to the public to copy and use this software without charge, 
provided that this Notice and any statement of authorship are reproduced 
on all copies. Neither the Government nor the University makes any 
warranty, express or implied, or assumes any liability or responsibility 
for the use of this software.
*/

/* $Id: seqlist.c,v 1.2 96/09/11 13:33:40 arve Exp $ */

//++pac
#ifdef _PLAN9
#include <bsd.h>
#else
#include <malloc.h>
#endif
#include <stdio.h>
#include <string.h>
#include "seqlist.h"

seqlist*
sl_prepend(char *name, char *s, seqlist* l)
{
  seqlist *tmp;

  tmp = (seqlist *) malloc(sizeof(seqlist));
  if (tmp == NULL) {
    return NULL;
  } else {
    tmp->name = name;
    tmp->seq = s;
    tmp->next = l;
    return tmp; 
  }   
}


void 
sl_free_list(seqlist* list)
{
  seqlist* tmp;

  tmp = list;
  while (tmp != NULL) {
    list = list->next;
    free(tmp);
    tmp = list;
  }
}

void 
sl_free_list_and_sequences(seqlist* list)
{
  seqlist* tmp;

  tmp = list;
  while (tmp != NULL) {
    list = list->next;
    free(tmp->name);
    free(tmp->seq);
    free(tmp);
    tmp = list;
  }
}


int 
sl_list_length(seqlist* list)
{
  int i=0;

  while (list != NULL) {
    i++;
    list = list->next;
  }
  return i;
}


seqlist*
sl_get_sequence_list(SEQFILE* sfp, seqlist* l)
{
  static int sequence_number=0;
  int status;
  char *s, *name;
  char tmpname[11];
  int len;
  SEQINFO *si;
  seqlist *tmp;

  tmp = l;
  status = seqfread(sfp, 0);	/* Read next sequence */
  while (status == 0) {		/* While reading OK */
    s = seqfrawseq(sfp, &len, 1); /* Return a new malloced buffer */
    si = seqfinfo(sfp, 0);	/* Use internal buffer for return value. */
    name = seqfmainid(sfp, 1); /* A good name, if it exists. */
    if (name == NULL) {
      sprintf(tmpname, "anon%d", sequence_number);
      sequence_number++;
      name = strdup(tmpname);
    }
    tmp = sl_prepend(name,
		     s,		/* The sequence */
		     tmp);	/* next element */
    status = seqfread(sfp, 0);	/* Get next sequence */
  }

  return tmp;

}



seqlist*
sl_copy_list(seqlist *l)
{
  seqlist *tmp;
  
  tmp = NULL;
  while (l != NULL) {
    tmp = sl_prepend(strdup(l->name), strdup(l->seq), tmp);
    l = l->next;
  }
  return tmp;
}

seqlist*
sl_find_name(seqlist *l, char *name)
{
  seqlist *tmp;

  for (tmp = l; tmp != NULL; tmp = tmp->next) {
    if (strcmp(tmp->name, name) == 0) {
      return tmp;
    }
  }
  
  return NULL;
}


void
sl_remove_prefix(seqlist *l, int free_string)
{
  seqlist *cur;
  char *name;
  
  for(cur=l; cur != NULL; cur = cur->next) {
    name = strchr(cur->name, ':');
    if (name != NULL) {
      name = strdup(name+1);	/* keep the name beyond the ':' */
      if (free_string) {
	free(cur->name);
      }
      cur->name = name;
    }
  }      
}

void
sl_check_name_length(seqlist *l)
{
  seqlist *cur;
  char *name;

  for(cur=l; cur != NULL; cur = cur->next) {
    name = cur->name;
    if (strlen(name) > 10) {
      name[11] = '\0';
    }
  }
}

