/*
Copyright, 1994-1995, The Regents of the University of California. 
This software was produced under U.S. Government contract(XXXXX)
by Los Alamos National Laboratory, which is operated by the University 
of California for the U.S. Department of Energy. The U.S Government 
is licensed to use, reproduce, and distribute this software. Permission 
is granted to the public to copy and use this software without charge, 
provided that this Notice and any statement of authorship are reproduced 
on all copies. Neither the Government nor the University makes any 
warranty, express or implied, or assumes any liability or responsibility 
for the use of this software.
*/

/* $Id: seqlist.h,v 1.2 96/09/11 13:33:41 arve Exp $ */

#ifndef SEQLIST_H
#define SEQLIST_H

#include "seqio.h"

typedef struct _seqlist {
  char *name;
  char *seq;
  struct _seqlist *next;
} seqlist;

/* 
   These macros are provided for backward compatibility.
 */
#define gb_seq_length(A) strlen(A->seq)
#define gb_next_entry(A) A->next
#define gb_seq_site(A,I) A->seq[i]
#define gb_locus(A)      A->name
#define gb_find_locus(A,B) sl_find_name(A,B)

seqlist* sl_prepend(char *name, char* s, seqlist* list);
/* 
   Allocates a new list node for the sequence s and sets 
   the next pointer to list. If an error occurs, NULL is returned.
   Note that s is not copied.
 */

void sl_free_list(seqlist* list);
/* 
   Free all list nodes but save the sequences and names for 
   other purposes.
 */

void sl_free_list_and_sequences(seqlist* list);
/* 
   Free the whole list including the character string for sequences
   and names.
 */

int sl_list_length(seqlist* list);
/* 
   Return the number of elements in the list.
 */

seqlist* sl_get_sequence_list(SEQFILE* sfp, seqlist* list);
/* 
   Read sequences from sfp and prepend them to list.
 */

seqlist* sl_copy_list(seqlist *l);
/* 
   Return a copy of l. The copy will be reversed.
   All strings are duplicated.
 */


seqlist* sl_find_name(seqlist *l, char *name);
/* 
   Finds the item with name name in list l.
   If name is notin the list, NULL is returned.
 */

void sl_remove_prefix(seqlist *l, int free_string);
/* 
   The SeqIO package introduces a prefix on the sequence identifier. 
   This may cause the identifiers to be too long for some programs 
   (e.g. neighbor in PHYLIP). This function removes this prefix.

   The identifier (after the prefix) is duplicated, and with the 
   flag free_string (1 == yes, 0 == no) you decide if the original 
   identifier should be freed.
 */

void sl_check_name_length(seqlist *l);
/* 
   If the sequence identifiers (with or without a SeqIO prefix) are 
   too long, strange things may happen in some programs (e.g. 
   neighbor in PHYLIP). This function allows you to brutally shorten
   these identifiers. No check is made that the identifiers are unique.
 */

#endif
