/*
Copyright, 1994-1995, The Regents of the University of California. 
This software was produced under U.S. Government contract(XXXXX)
by Los Alamos National Laboratory, which is operated by the University 
of California for the U.S. Department of Energy. The U.S Government 
is licensed to use, reproduce, and distribute this software. Permission 
is granted to the public to copy and use this software without charge, 
provided that this Notice and any statement of authorship are reproduced 
on all copies. Neither the Government nor the University makes any 
warranty, express or implied, or assumes any liability or responsibility 
for the use of this software.
*/

/* $Id: struct_methods.c,v 1.2 96/09/11 13:33:43 arve Exp $ */

//++pac
#ifndef _PLAN9
#include <malloc.h>
#endif
#include <stdio.h>
#include <string.h>

#include "structures.h"
#include "struct_methods.h"

/* Initialize a sequence name list */

void
init_seq_name_list(seqnamelist *l) {
  l->first = NULL;
  l->last = NULL;
  l->nr_of_items = 0;
}


void 
append_seq_name(seqnamelist* l, char *name) {
  seqnameitem *tmp;
  
  if ((tmp = (seqnameitem*) malloc(sizeof(seqnameitem)))) {
    tmp->next = NULL;
    if (!(strncpy(tmp->seqname, name, LOCUSLENGTH))) {
      fprintf(stderr, "Not enough memory!\n");
      exit(2);
    } 
    if (l->last) {
      l->last->next = tmp;
      l->last = tmp;
      l->nr_of_items++;
    } else {
      l->first = tmp;
      l->last = tmp;
      l->nr_of_items = 1;
    }
  } else {
    fprintf(stderr, "Not enough memory!\n");
    exit(2);
  }
}


seqnameitem*
first_seq_name(seqnamelist *l) {
  if (l->nr_of_items == 0) {
    return NULL;		/* Could be replaced by an error exit */
  } else {
    return l->first;
  }
}


/* 
 * This function returns the index of secuence <name> in the list or -1 
 * if the list is empty and -2 if the seq does not exist in the list.
 */

int
item_index(seqnamelist *snl,char *name) {
  seqnameitem *current;
  unsigned i=0;

  if (snl->nr_of_items == 0) {
    return -1;
  } else {
    for(current=first_seq_name(snl); current!=NULL; current=current->next,i++){
      if (!strcmp(current->seqname, name)) {
	return i;		/* We found the index */
      }
    }
    return -2;			/* <name> weren't in the list */
  }
}


/*
 * Remove every element in the list, but leave the
 * list head.
 */

void delete_list_items(seqnamelist *l) {
  seqnameitem *i, *tmp;
  
  i = l->first; 
  while (i != NULL) {
    tmp = i;
    i= i->next;
    free(tmp);
  }
  l->first = NULL;
  l->last = NULL;
  l->nr_of_items = 0;
}

/*
 * Remove the list, including list head.
 */

void delete_list(seqnamelist *l) {
    
    if (l) {
	delete_list_items(l);
	free (l);
    } else {
	/* Nothing */
    }
};

differencelist*
append_diff(differencelist* l, 
	    seqnameitem *s0, seqnameitem *s1, seqnameitem *s2, seqnameitem *s3, 	    float diff, float std_dev) {
  differencelist *tmp;

  if ((tmp = (differencelist*) malloc(sizeof(differencelist)))) {
    tmp->item[0] = s0;
    tmp->item[1] = s1;
    tmp->item[2] = s2;
    tmp->item[3] = s3;
    tmp->diff = diff;
    tmp->std_dev = std_dev;
  } else {
    fprintf(stderr, "Out of memory.\n");
    exit(1);
  }
  
  if (l) {
    tmp->next = l;
  } else {
    tmp->next = NULL;
  }

  return tmp;
}

char*
diff_item(differencelist *l, unsigned i) {
  if (l) {
    return l->item[i]->seqname;
  } else {
    return NULL;
  }
}


float
diff_val(differencelist *l) {
  if (l) {
    return l->diff;
  } else {
    fprintf(stderr, "Programming error: Tried to access empty differencelist.\n");
    exit(1);
  }
}


float
sd_val(differencelist *l) {
  if (l) {
    return l->std_dev;
  } else {
    fprintf(stderr, "Programming error: Tried to access empty differencelist.\n");
    exit(1);
  }
}


differencelist*
diff_next(differencelist *l) {
  if (l) {
    return l->next;
  } else {
    fprintf(stderr, "Programming error: Tried to access empty difference list.\n");
    exit(1);
  }
}
